package at.bestsolution.rsd.rt.client.ws;

import java.io.IOException;
import java.net.URI;

import javax.inject.Inject;
import javax.inject.Named;
import javax.websocket.DeploymentException;
import javax.websocket.WebSocketContainer;

import at.bestsolution.rsd.rt.rpc.RPCMessageBuilder;
import at.bestsolution.rsd.rt.rpc.RPCMessageParser;

public abstract class BaseWSClient<T extends BaseWSClientEndpoint> {
	private WebSocketContainer container;
	private String endpointHost;
	private String endpointPath;
	private RPCMessageBuilder messageBuilder;
	private RPCMessageParser messageParser;
	
	private T endpoint;

	@Inject
	public void setWebSocketContainer(WebSocketContainer container) {
		this.container = container;
	}

	@Inject
	public void setEndpointHost(@Named("endpointHost") String endpointHost) {
		this.endpointHost = endpointHost;
	}

	@Inject
	public void setEndpointPath(@Named("endpointPath") String endpointPath) {
		this.endpointPath = endpointPath;
	}
	
	@Inject
	public void setRPCMessageBuilder(RPCMessageBuilder messageBuilder) {
		this.messageBuilder = messageBuilder;
	}
	
	protected RPCMessageBuilder getMessageBuilder() {
		return messageBuilder;
	}
	
	@Inject
	public void setRPCMessageParser(RPCMessageParser messageParser) {
		this.messageParser = messageParser;
	}
	
	protected RPCMessageParser getMessageParser() {
		return messageParser;
	}

	public T getEndpoint() {
		if (endpoint != null) {
			return endpoint;
		}
		synchronized (this) {
			if (endpoint == null) {
				T localEndpoint = createEndpoint();

				try {
					container.connectToServer(localEndpoint, URI.create("ws://" + endpointHost + "/" + endpointPath));
					this.endpoint = localEndpoint;
				} catch (DeploymentException | IOException e) {
					throw new IllegalStateException(e);
				}
			}
		}
		return endpoint;
	}
	
	protected abstract T createEndpoint();
}
